from interp.grid import delaunay_grid as basegrid
from interp.tools import exact_func_3D, log

import numpy as np


class grid(basegrid):
  def __init__(self, verts, q):
    basegrid.__init__(self, verts, q)

  def for_qhull_generator(self):
    """
      this returns a generator that should be fed into qdelaunay
    """

    yield '3';
    yield '%d' % len(self.verts)

    for p in self.verts:
      yield "%f %f %f" % tuple(p)

  def for_qhull(self):
    """
      this returns a single string that should be fed into qdelaunay
    """
    r  = '3\n'
    r += '%d\n' % len(self.verts)
    for p in self.verts:
      r += "%f %f %f\n" % tuple(p)
    return r

class rect_grid(grid):
  def __init__(self, xres = 5, yres = 5, zres = 5):
    xmin = -1.0
    xmax =  1.0
    xspan = xmax - xmin
    xdel = xspan / float(xres - 1)

    ymin = -1.0
    ymay =  1.0
    yspan = ymay - ymin
    ydel = yspan / float(yres - 1)

    zmin = -1.0
    zmaz =  1.0
    zspan = zmaz - zmin
    zdel = zspan / float(zres - 1)


    verts = []
    q      = []
    for x in xrange(xres):
      cur_x = xmin + (x * xdel)
      for y in xrange(yres):
        cur_y = ymin + (y * ydel)
        for z in xrange(zres):
          cur_z = zmin + (z * zdel)
          verts.append([cur_x, cur_y, cur_z])
          q.append(exact_func_3D((cur_x, cur_y, cur_z)))
    grid.__init__(self, verts, q)
  # self.construct_connectivity()

  def for_qhull_generator(self):
    """
      this returns a generator that should be fed into qdelaunay
    """

    yield '3';
    yield '%d' % len(self.verts)

    for p in self.verts:
      yield "%f %f %f" % tuple(p)

  def for_qhull(self):
    """
      this returns a single string that should be fed into qdelaunay
    """
    r  = '3\n'
    r += '%d\n' % len(self.verts)
    for p in self.verts:
      r += "%f %f %f\n" % tuple(p)
    return r

class random_grid(rect_grid):
  def __init__(self, num_verts = 10):
    verts = []
    q = []

    r = np.random

    appx_side_res = int(np.power(num_verts, 1/3.0))
    log.debug("appx_side_res: %d" % appx_side_res)
    delta = 1.0 / float(appx_side_res)

    for x in xrange(appx_side_res + 1):
      pass

    for i in xrange(num_verts):
      cur_x = r.rand()
      cur_y = r.rand()
      cur_z = r.rand()

      verts.append([cur_x, cur_y, cur_z])
      q.append(exact_func_3D((cur_x, cur_y, cur_z)))
    grid.__init__(self, verts, q)

    self.verts = np.array(self.verts)
    self.q      = np.array(self.q)
